==> iRRAM_2004_01.tar.gz: iRRAM_2004_01/include/iRRAM.h <==
/*

iRRAM.h -- central header file for the iRRAM library
 
Copyright (C) 2001-2003 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/


/*
Authors:  all by Norbert, except: 

  2003-07 INTERVAL extensions partly by Shao Qi

  2001-07 INTEGER and RATIONAL partly by Tom van Diessen
  

*/

#ifndef iRRAM_H
#define iRRAM_H


#if defined(LRGMP)
	#include "LRGMP_interface.h"
	#define iRRAM_BACKEND "LRGMP"
#elif defined(GMP)
	#include "GMP_interface.h"
	#define iRRAM_BACKEND "GMP"
#elif defined(MPFR)
	#include "MPFR_interface.h"
	#define iRRAM_BACKEND "MPFR"
#else
	#include "Backend not defined"
#endif

extern "C" void iRRAM_initialize(int argc,char** argv);

template <class ARGUMENT, class RESULT> 
 RESULT iRRAM_exec (RESULT f(ARGUMENT), ARGUMENT);

#ifndef do_not_activate_iRRAM_control
extern void compute(void); 

long iRRAM_compute(long arg){compute();return 0L;};
 
int main (int argc,char **argv){
  iRRAM_initialize(argc,argv);
  long erg=iRRAM_exec(iRRAM_compute,0L);
}
#endif

// forward declaration of some classes

class INTEGER;
class RATIONAL;
class DYADIC;
class LAZY_BOOLEAN;

class METRIC_OBJECT;
class REAL;
class COMPLEX;


/****************************************************************************/
// Definition of class LAZY_BOOLEAN 
/****************************************************************************/
#define BOTTOM -1
class LAZY_BOOLEAN
{
public:
LAZY_BOOLEAN(){value=false;};
LAZY_BOOLEAN(int b){value=b;};
friend LAZY_BOOLEAN operator && (const LAZY_BOOLEAN& x, const LAZY_BOOLEAN& y);
friend LAZY_BOOLEAN operator || (const LAZY_BOOLEAN& x, const LAZY_BOOLEAN& y);
friend LAZY_BOOLEAN operator ! ( const LAZY_BOOLEAN& x);
friend int choose(const LAZY_BOOLEAN& x1= false,
                  const LAZY_BOOLEAN& x2= false,
                  const LAZY_BOOLEAN& x3= false,
                  const LAZY_BOOLEAN& x4= false,
                  const LAZY_BOOLEAN& x5= false,
                  const LAZY_BOOLEAN& x6= false );
operator bool() const;
private:
int value;
};


/****************************************************************************/
// Definition of class METRIC_OBJECT 
/****************************************************************************/

#define SIZETYPEMANTISSA unsigned long
#define SIZETYPEEXPONENT long

extern unsigned long int METRIC_OBJECT_COUNT;

struct sizetype { SIZETYPEMANTISSA  mantissa; SIZETYPEEXPONENT exponent; } ;

class METRIC_OBJECT {
public:
METRIC_OBJECT(){
METRIC_OBJECT_COUNT+=1;      
/*fprintf(stderr," +- Erzeuge: %x\n",this);*/
};

virtual ~METRIC_OBJECT(){
/*fprintf(stderr," +- Lsche: %x\n",this);*/
  METRIC_OBJECT_COUNT-=1; 
}
;
virtual void adderror (sizetype error){};
virtual void seterror (sizetype error){};
virtual void geterror (sizetype& error) const{};
};


#include "iRRAM_REAL.h"
#include "iRRAM_DYADIC.h"
#include "iRRAM_INTEGER.h"
#include "iRRAM_RATIONAL.h"







void rscanf  (const char *format, void * input);
void rprintf (const char *rformat, ...);

void continous_begin ();
void continous_end   ();


#define ABSOLUTE 0
#define RELATIVE 1
void precision_policy (long policy);

void stiff_begin ();
void stiff_end   ();


REAL strtoREAL(char* s, char** endptr);
REAL atoREAL(char* s);

/****************************************************************************/
// Definition of class REALMATRIX
/****************************************************************************/

class REALMATRIX : public METRIC_OBJECT
{
public:
// Constructors: -------------------------------

REALMATRIX(unsigned int rows,unsigned int columns);
REALMATRIX();
REALMATRIX(const REALMATRIX& y);
// Copy Constructor: ---------------------------

REALMATRIX&   operator = (const REALMATRIX& y);

// Destructor: ---------------------------------

~REALMATRIX();

// Access to matrix elements: ------------------

REAL&   operator ()           (unsigned int row,
                               unsigned int column) const;

REAL&   element               (unsigned int row,
                               unsigned int column) const;

// Standard Arithmetic: ------------------------

friend REALMATRIX  operator + (const REALMATRIX& x,
                               const REALMATRIX& y);
friend REALMATRIX  operator - (const REALMATRIX& x,
                               const REALMATRIX& y);
friend REALMATRIX  operator * (const REALMATRIX& x,
                               const REALMATRIX& y);
friend REALMATRIX  operator / (const REALMATRIX& x,
                               const REALMATRIX& y);

// Arithmetic with Scalar: ---------------------
friend REALMATRIX  operator * (const REALMATRIX& x,
                               const REAL& y);
inline friend REALMATRIX operator * (const REAL& y,
                               const REALMATRIX& x)
       {return x*y;};

friend REALMATRIX  operator / (const REALMATRIX& x,
                               const REAL& y);

// Information on Dimensions: ------------------

friend inline unsigned int rows (const REALMATRIX& x)
                {return x.maxrow;};
friend inline unsigned int columns (const REALMATRIX& x)
                {return x.maxcolumn;};

friend int bound (const REALMATRIX& x, 
                  const long k);

// Linear Algebra: -----------------------------

friend REALMATRIX eye     (unsigned int rows);
friend REALMATRIX zeroes  (unsigned int rows,
                           unsigned int columns);
friend REALMATRIX ones    (unsigned int rows,
                           unsigned int columns);
friend REAL       maxnorm (const REALMATRIX& x);
friend REAL       rowsumnorm (const REALMATRIX& x);
friend REAL       colsumnorm (const REALMATRIX& x);

friend REALMATRIX solve (
       REALMATRIX& lside,
       REALMATRIX& rside,
       int use_pivot);

// IO-routines: --------------------------

friend void       rwrite  (const REALMATRIX& x,
                           const long p,
                           const long w);
friend void       rwrite  (const REALMATRIX& x,
                           const long w);
friend void       rshow   (const REALMATRIX& x,
                           const long w);

// Limits: --------------------------

friend REALMATRIX limit_lip (REALMATRIX f(long, const REALMATRIX&),
                           long lip,
                           bool on_domain(const REALMATRIX&),
                           const REALMATRIX& x);


public:
REAL*  values; 
unsigned int    maxrow,maxcolumn;
void adderror (sizetype error);
void seterror (sizetype error);
void geterror (sizetype& error) const;
};

class SPM_ELEMENT
{ public:
REAL value;
unsigned int rowindex;
unsigned int colindex;
SPM_ELEMENT* nextcol;
SPM_ELEMENT* nextrow; 
};

/****************************************************************************/
// Definition of class SPARSEREALMATRIX
/****************************************************************************/

class SPARSEREALMATRIX : public METRIC_OBJECT
{
public:
// Constructors: -------------------------------

SPARSEREALMATRIX(unsigned int rows,unsigned int columns);
SPARSEREALMATRIX();
SPARSEREALMATRIX(const SPARSEREALMATRIX& y);
// Copy Constructor: ---------------------------

SPARSEREALMATRIX&   operator = (const SPARSEREALMATRIX& y);

// Destructor: ---------------------------------

~SPARSEREALMATRIX();
void clear();
void free();

// Access to matrix elements: ------------------

friend void sparse_set(SPARSEREALMATRIX& m,unsigned int i,unsigned int j,const REAL& x);
friend void sparse_unset(SPARSEREALMATRIX& m,unsigned int i,unsigned int j);
friend SPM_ELEMENT* sparse_get_ptr(SPARSEREALMATRIX& m,unsigned int i,unsigned int j);

REAL    operator ()           (unsigned int row,
                               unsigned int column);


// Standard Arithmetic: ------------------------

friend SPARSEREALMATRIX  operator + (const SPARSEREALMATRIX& x,
                               const SPARSEREALMATRIX& y);
friend SPARSEREALMATRIX  operator - (const SPARSEREALMATRIX& x,
                               const SPARSEREALMATRIX& y);
friend SPARSEREALMATRIX  operator * (const SPARSEREALMATRIX& x,
                               const SPARSEREALMATRIX& y);
friend SPARSEREALMATRIX  operator / (const SPARSEREALMATRIX& x,
                               const SPARSEREALMATRIX& y);

// Arithmetic with Scalar: ---------------------
friend SPARSEREALMATRIX  operator * (const SPARSEREALMATRIX& x,
                               const REAL& y);
inline friend SPARSEREALMATRIX operator * (const REAL& y,
                               const SPARSEREALMATRIX& x)
       {return x*y;};

friend SPARSEREALMATRIX  operator / (const SPARSEREALMATRIX& x,
                               const REAL& y);

// Information on Dimensions: ------------------

friend inline unsigned int rows (const SPARSEREALMATRIX& x)
                {return x.maxrow;};
friend inline unsigned int columns (const SPARSEREALMATRIX& x)
                {return x.maxcolumn;};

friend int bound (const SPARSEREALMATRIX& x, 
                  const long k);

// Linear Algebra: --------------------------

friend SPARSEREALMATRIX sparse_eye     (unsigned int rows);
friend SPARSEREALMATRIX sparse_zeroes  (unsigned int rows,
                           unsigned int columns);
friend SPARSEREALMATRIX sparse_ones    (unsigned int rows,
                           unsigned int columns);
friend REAL       maxnorm (const SPARSEREALMATRIX& x);
friend REAL       rowsumnorm (const SPARSEREALMATRIX& x);
friend REAL       colsumnorm (const SPARSEREALMATRIX& x);

friend SPARSEREALMATRIX solve (
       SPARSEREALMATRIX& lside,
       SPARSEREALMATRIX& rside,
       int use_pivot);


// IO-routines: --------------------------

friend void       rwrite  (const SPARSEREALMATRIX& x,
                           const long p,
                           const long w);
friend void       rwrite  (const SPARSEREALMATRIX& x,
                           const long w);
friend void       rshow   (const SPARSEREALMATRIX& x,
                           const long w);

// Limits: --------------------------

friend SPARSEREALMATRIX limit_lip (SPARSEREALMATRIX f(long, const SPARSEREALMATRIX&),
                           long lip,
                           const SPARSEREALMATRIX& x);

//friend SPARSEREALMATRIX limit_mv (SPARSEREALMATRIX f(long prec,
//                                         long* choice,
//                                         const SPARSEREALMATRIX&),
//                           const SPARSEREALMATRIX& x);

//private:
SPM_ELEMENT**  rowvector; 
SPM_ELEMENT**  colvector; 
SPM_ELEMENT* hotspot;
unsigned int    maxrow,maxcolumn,filled;
void adderror (sizetype error);
void seterror (sizetype error);
void geterror (sizetype& error) const;
};
 

/****************************************************************************/
// Definition of class INTERVAL
// The syntax of the functions is chosen similar to 
//    SUN C++ Interval Arithmetic Programming Reference
// The semantics must of course is according to the neccessities of TTE
// Not all functions have been implemented!
/****************************************************************************/
class INTERVAL 
{
public:

// Constructors: -------------------------------

INTERVAL();

INTERVAL(const REAL& x);
INTERVAL(const REAL& l, const REAL& r);

// Standard Arithmetic: ------------------------

friend INTERVAL  operator +  (const INTERVAL& x,
                             const INTERVAL& y);
friend INTERVAL  operator -  (const INTERVAL& x,
                             const INTERVAL& y);
friend INTERVAL  operator -  (const INTERVAL& x);
friend INTERVAL  operator *  (const INTERVAL& x,
                             const INTERVAL& y);
friend INTERVAL  operator /  (const INTERVAL& x,
                             const INTERVAL& y);

friend REAL wid(const INTERVAL& x);
friend REAL inf(const INTERVAL& x);
friend REAL sup(const INTERVAL& x);
friend REAL mid(const INTERVAL& x);
friend REAL mag(const INTERVAL& x);
friend REAL mig(const INTERVAL& x);

friend LAZY_BOOLEAN superset (const INTERVAL& x,
                             const INTERVAL& y);
friend LAZY_BOOLEAN proper_superset (const INTERVAL& x,
                             const INTERVAL& y);
friend LAZY_BOOLEAN subset (const INTERVAL& x,
                             const INTERVAL& y);
friend LAZY_BOOLEAN proper_subset (const INTERVAL& x,
                             const INTERVAL& y);
friend LAZY_BOOLEAN in_interior (const INTERVAL& x,
                             const INTERVAL& y);
friend LAZY_BOOLEAN disjoint (const INTERVAL& x,
                             const INTERVAL& y);
friend LAZY_BOOLEAN in (const REAL& x,
                             const INTERVAL& y);

friend INTERVAL interval_hull (const INTERVAL& x,
                             const INTERVAL& y);
friend INTERVAL intersect (const INTERVAL& x,
                             const INTERVAL& y);

friend INTERVAL fabs(const INTERVAL& x);

friend INTERVAL exp(const INTERVAL& x);
friend INTERVAL log(const INTERVAL& x);
friend INTERVAL sin(const INTERVAL& x);
friend INTERVAL cos(const INTERVAL& x);
friend INTERVAL tan(const INTERVAL& x);
friend INTERVAL asin(const INTERVAL& x);
friend INTERVAL acos(const INTERVAL& x);
friend INTERVAL atan(const INTERVAL& x);

private :
REAL low;
REAL upp;
};


/****************************************************************************/
// Definition of class COMPLEX 
/****************************************************************************/
#include "iRRAM_COMPLEX.h"

/****************************************************************************/
// arithmetic functions
/****************************************************************************/
REAL power(const REAL& x, const REAL& y);
REAL power(const REAL& x, long n);
REAL power(const REAL& x, int n);
REAL modulo  (const REAL& x, const REAL& y);
REAL maximum (const REAL& x, const REAL& y);
REAL minimum (const REAL& x, const REAL& y);

/****************************************************************************/
// roots
/****************************************************************************/
REAL sqrt    (const REAL& x);
REAL root    (const REAL& x,long n);

/****************************************************************************/
// trigonometric functions
/****************************************************************************/
REAL sin     (const REAL& x);
REAL cos     (const REAL& x);
REAL tan     (const REAL& x);
REAL cotan   (const REAL& x);
REAL sec     (const REAL& x);
REAL cosec   (const REAL& x);

/****************************************************************************/
// inverse trigonometric functions
/****************************************************************************/
REAL atan    (const REAL& x);
REAL asin    (const REAL& x);
REAL acos    (const REAL& x);
REAL acotan  (const REAL& x);
REAL asec    (const REAL& x);
REAL acosec  (const REAL& x);

/****************************************************************************/
//hyperbolic functions
/****************************************************************************/
REAL sinh    (const REAL& x);
REAL cosh    (const REAL& x);
REAL tanh    (const REAL& x);
REAL coth    (const REAL& x);
REAL sech    (const REAL& x);
REAL cosech  (const REAL& x);

/****************************************************************************/
// inverse hyperbolic functions 
/****************************************************************************/
REAL asinh   (const REAL& x);
REAL acosh   (const REAL& x);
REAL atanh   (const REAL& x);
REAL acoth   (const REAL& x);
REAL asech   (const REAL& x);
REAL acosech (const REAL& x);

/****************************************************************************/
// exponentiation + logarithm
/****************************************************************************/
REAL exp     (const REAL& x);
REAL log     (const REAL& x);

/****************************************************************************/
// special constants values                                                 
/****************************************************************************/
REAL pi      ();   // = 3.141592653...
REAL euler   ();   // = 2.718281828...
REAL ln2     ();   // = 0.693147180...

/****************************************************************************/
//  matrix functions
/****************************************************************************/
REALMATRIX exp    (const REALMATRIX& x);

REALMATRIX steady_state (const REALMATRIX& x);

REALMATRIX steady_state (const SPARSEREALMATRIX& x);
REALMATRIX equilib (const SPARSEREALMATRIX& x);
void equilib_del (SPARSEREALMATRIX& x,REALMATRIX& z);




/****************************************************************************/
// templates for limit operators
/****************************************************************************/

template <class ARGUMENT, class RESULT> 
RESULT limit_mv (RESULT f(long prec,
                                         long* choice,
                                         const ARGUMENT&),
                           const ARGUMENT& x);

template <class ARGUMENT, class RESULT> 
RESULT limit (RESULT f(long prec,const ARGUMENT&),
                           const ARGUMENT& x);


template <class ARGUMENT, class RESULT> 
RESULT limit (RESULT f(long prec,const ARGUMENT&, long),
                           const ARGUMENT& x, long param);




extern char* iRRAM_error_msg[];
class iRRAM_Numerical_Exception
{
public:
iRRAM_Numerical_Exception(const long msg){type=msg;};
//private:
long type;
};
#define ERRORDEFINE(x,y) x,
enum iRRAM_execption_list {
#include "iRRAM_errno.h"
};




#endif
/* ! iRRAM_H */

==> iRRAM_2004_02.tar.gz: iRRAM_2004_02/include/iRRAM.h <==
/*

iRRAM.h -- central header file for the iRRAM library
 
Copyright (C) 2001-2003 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/


/*
Authors:  all by Norbert, except: 

  2003-07 INTERVAL extensions partly by Shao Qi

  2001-07 INTEGER and RATIONAL partly by Tom van Diessen
  

*/

#ifndef iRRAM_H
#define iRRAM_H

#include <string>
using namespace std;


// forward declaration of some classes

class INTEGER;
class RATIONAL;
class DYADIC;
class LAZY_BOOLEAN;
class REAL;
class COMPLEX;

#if defined(LRGMP)
	#include "LRGMP_interface.h"
	#define iRRAM_BACKEND "LRGMP"
#elif defined(cGMP)
	#include "cGMP_interface.h"
	#define iRRAM_BACKEND "cGMP"
#elif defined(GMP)
	#include "GMP_interface.h"
	#define iRRAM_BACKEND "GMP"
#elif defined(MPFR)
	#include "MPFR_interface.h"
	#define iRRAM_BACKEND "MPFR"
#elif defined(INTMPF)
	#include "INTMPF_interface.h"
	#define iRRAM_BACKEND "INTMPF"
#else
	#include "Backend not defined"
#endif


extern "C" void iRRAM_initialize(int argc,char** argv);

template <class ARGUMENT, class RESULT> 
 RESULT iRRAM_exec (RESULT f(ARGUMENT), ARGUMENT);

#ifndef do_not_activate_iRRAM_control
extern void compute(void); 

int iRRAM_compute(int arg){compute();return 0;};
 
int main (int argc,char **argv){
  iRRAM_initialize(argc,argv);
  return iRRAM_exec(iRRAM_compute,0);
}
#endif


#define SIZETYPEMANTISSA unsigned int
#define SIZETYPEEXPONENT int

struct sizetype { SIZETYPEMANTISSA  mantissa; SIZETYPEEXPONENT exponent; } ;



#include "iRRAM_LAZYBOOLEAN.h"
#include "iRRAM_REAL.h"
#include "iRRAM_DYADIC.h"
#include "iRRAM_INTEGER.h"
#include "iRRAM_RATIONAL.h"
#include "iRRAM_REALMATRIX.h"
#include "iRRAM_SPARSEREALMATRIX.h"
#include "iRRAM_COMPLEX.h"
#include "iRRAM_INTERVAL.h"
#include "iRRAM_STREAMS.h"




#define ABSOLUTE 0
#define RELATIVE 1
void precision_policy (int policy);
void stiff_begin ();
void stiff_end   ();
void continous_begin ();
void continous_end   ();





REAL atoREAL(const char* s);


 

/****************************************************************************/
// arithmetic functions
/****************************************************************************/
REAL power(const REAL& x, const REAL& y);
REAL power(const REAL& x, int n);
REAL modulo  (const REAL& x, const REAL& y);
REAL maximum (const REAL& x, const REAL& y);
REAL minimum (const REAL& x, const REAL& y);

/****************************************************************************/
// roots
/****************************************************************************/
REAL sqrt    (const REAL& x);
REAL root    (const REAL& x,int n);

/****************************************************************************/
// trigonometric functions
/****************************************************************************/
REAL sin     (const REAL& x);
REAL cos     (const REAL& x);
REAL tan     (const REAL& x);
REAL cotan   (const REAL& x);
REAL sec     (const REAL& x);
REAL cosec   (const REAL& x);

/****************************************************************************/
// inverse trigonometric functions
/****************************************************************************/
REAL atan    (const REAL& x);
REAL asin    (const REAL& x);
REAL acos    (const REAL& x);
REAL acotan  (const REAL& x);
REAL asec    (const REAL& x);
REAL acosec  (const REAL& x);

/****************************************************************************/
//hyperbolic functions
/****************************************************************************/
REAL sinh    (const REAL& x);
REAL cosh    (const REAL& x);
REAL tanh    (const REAL& x);
REAL coth    (const REAL& x);
REAL sech    (const REAL& x);
REAL cosech  (const REAL& x);

/****************************************************************************/
// inverse hyperbolic functions 
/****************************************************************************/
REAL asinh   (const REAL& x);
REAL acosh   (const REAL& x);
REAL atanh   (const REAL& x);
REAL acoth   (const REAL& x);
REAL asech   (const REAL& x);
REAL acosech (const REAL& x);

/****************************************************************************/
// exponentiation + logarithm
/****************************************************************************/
REAL exp     (const REAL& x);
REAL log     (const REAL& x);

/****************************************************************************/
// special constants values                                                 
/****************************************************************************/
REAL pi      ();   // = 3.141592653...
REAL euler   ();   // = 2.718281828...
REAL ln2     ();   // = 0.693147180...

/****************************************************************************/
//  matrix functions
/****************************************************************************/
REALMATRIX exp    (const REALMATRIX& x);

REALMATRIX steady_state (const REALMATRIX& x);

REALMATRIX steady_state (const SPARSEREALMATRIX& x);
REALMATRIX equilib (const SPARSEREALMATRIX& x);
void equilib_del (SPARSEREALMATRIX& x,REALMATRIX& z);




/****************************************************************************/
// templates for limit operators
/****************************************************************************/

template <class ARGUMENT, class RESULT> 
RESULT limit_mv (RESULT f(int prec,
                                         int* choice,
                                         const ARGUMENT&),
                           const ARGUMENT& x);

template <class ARGUMENT, class RESULT> 
RESULT limit (RESULT f(int prec,const ARGUMENT&),
                           const ARGUMENT& x);


template <class ARGUMENT, class RESULT> 
RESULT limit (RESULT f(int prec,const ARGUMENT&, int),
                           const ARGUMENT& x, int param);




extern char* iRRAM_error_msg[];
class iRRAM_Numerical_Exception
{
public:
iRRAM_Numerical_Exception(const int msg){type=msg;};
//private:
int type;
};
#define ERRORDEFINE(x,y) x,
enum iRRAM_execption_list {
#include "iRRAM_errno.h"
};




#endif
/* ! iRRAM_H */
